/*Author: dirk.lze@gmail.com
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#include "plugin_kateperlcheck.h"
#include "plugin_kateperlcheck.moc"
#include <qregexp.h>
#include <qstring.h>
#include <kactioncollection.h>
//Added by qt3to4:
#include <Q3CString>
#include <QApplication>
#include <kaction.h>
#include <kdebug.h>
#include <k3dockwidget.h>
#include <klocale.h>
#include <kcursor.h>
#include <kate/pluginconfigpageinterface.h>
#include <kate/plugin.h>
#include <QFile>
#include <Q3PopupMenu>
#include <QMenu>
#include <kgenericfactory.h>
#include <kateview.h>
#include <QTreeView>
#include <qmessagebox.h>
#include <KToolInvocation>
#include <KMenu>
#include <QTimer>

/**
* \brief ToolTip class for debugger Vars
*/

Tip::Tip(QWidget* parent ): QWidget (parent)
{
  setWindowFlags(Qt::ToolTip);
  m_Tip = new QTextBrowser();
  resize(200,100);
  QGridLayout* grid = new QGridLayout;
  grid->addWidget(m_Tip);
  grid->setContentsMargins ( 0,0,0,0);
  setLayout(grid);
}

/**
* \brief shows a ToolTip 
* \param QPoint p 
* \param QString str
*/

void Tip::showTip(QPoint p,QString str)
{
  if(str.length() == 0)
    return;

  m_Tip->setText(str);
  m_bLeave = false;

  if(!isVisible() )
    show();
  move (p.x()+5, p.y()+5);
}

/**
* \brief starts a singleShot to close the Tolltip
* \param QEvent * event unused
*/

void Tip::leaveEvent ( QEvent * event ) 
{
  Q_UNUSED(event);
  m_bLeave = true;
  QTimer::singleShot(5000, this, SLOT(hide()));
}

/**
* \brief called trough a singleShot in leaveEvent
* \see leaveEvent
*/

void Tip::hide ()
{
  if(m_bLeave)
    close();
}

/**
* \brief Manage a KProcess for the debugger.
*/

Perl::Perl(QObject* parent ): QObject (parent)
{
    m_Perl = new KProcess(this);
    m_Perl->setOutputChannelMode(KProcess::MergedChannels);
    connect( m_Perl, SIGNAL(readyReadStandardError()), this,SLOT(readOutput()));
    connect( m_Perl, SIGNAL(readyReadStandardOutput()),this,SLOT(readOutput()));    
}

void Perl::readOutput()
{
  while(m_Perl->canReadLine())
  {
     output +=  m_Perl->readLine();
  }
  emit readLine( output );
}

qint64 Perl::write(const char *data)
{
  output = "";
  return m_Perl->write(data);
}

void Perl::start()
{
  m_Perl->start();
}

void Perl::close()
{
  m_Perl->close();
}

/*
 * @short Kate Perl Plug-In, Perl syntax check (perl -c ), Perl::Critic and Debugger Plug-In.
 * @author Dirk Lindner <dirk.lze@gmail.com>
 * @version 1.9
 */

K_EXPORT_COMPONENT_FACTORY( kateperlcheckplugin, KGenericFactory<PluginKatePERLCheck>( "kateperlcheck" ) )

/**
* \brief Perl syntax check, perlcritic and perldebug pluginfor kate
*/

PluginKatePERLCheckView::PluginKatePERLCheckView(Kate::MainWindow *mainwin,SettingsDialog* sdialog)
    : Kate::PluginView (mainwin),KXMLGUIClient(),win(mainwin)
{
    Q_UNUSED(sdialog);
    dock = win->createToolView("kate_plugin_perlcheck_ouputview", Kate::MainWindow::Bottom, SmallIcon("application-x-perl"), i18n("Perl"));
    pdpView = new QPdbWidget(dock);

    aDebug = actionCollection()->addAction("debug_start");
    aDebug->setText(i18n("Debug"));
    aDebug->setIcon( KIcon ("debug-run") );
    aDebug->setToggleAction(true);
    connect(aDebug, SIGNAL(toggled(bool)), this, SLOT(debug(bool)));
 
    aReStart = actionCollection()->addAction("debug_restart");
    aReStart->setText(i18n("Restart"));
    aReStart->setIcon( KIcon("restart") );
    connect(aReStart, SIGNAL(triggered()), this, SLOT(debugRestart()));

    aStep = actionCollection()->addAction("debug_singleStep");
    aStep->setText(i18n("Single step"));
    aStep->setIcon( KIcon(KIcon ("debug-step-into-instruction")  ) );
    connect(aStep, SIGNAL(triggered()), this, SLOT(singleStep()));

    aReturnFS = actionCollection()->addAction("debug_returnFromSub");
    aReturnFS->setText(i18n("Return from sub"));
    aReturnFS->setIcon( KIcon( KIcon ("debug-step-out") ) );
    connect(aReturnFS, SIGNAL(triggered()), this, SLOT(returnFromSub()));

    aContinue = actionCollection()->addAction("debug_continueUntilPosition");
    aContinue->setText(i18n("Continue"));
    aContinue->setIcon( KIcon(KIcon ("debug-run-cursor")) );
    connect(aContinue, SIGNAL(triggered()), this, SLOT(continueUntilPosition()));

    aBreakPoint = actionCollection()->addAction("debug_setBreakpoint");
    aBreakPoint->setText(i18n("Breakpoint"));
    aBreakPoint->setIcon( KIcon("script-error") );
    aBreakPoint->setCheckable(true);
    connect(aBreakPoint, SIGNAL(triggered()), this, SLOT(addBreakPoint()));    

    aStepOver = actionCollection()->addAction("debug_stepOver");
    aStepOver->setText(i18n("Step over"));
    aStepOver->setIcon( KIcon("debug-step-over") );
    connect(aStepOver, SIGNAL(triggered()), this, SLOT(stepOver()));

    aDelAll = actionCollection()->addAction("debug_deleteAllBreakpoints");
    aDelAll->setText(i18n("Delete all Breakpoints"));
    aDelAll->setIcon( KIcon("delete") );
    connect(aDelAll, SIGNAL(triggered()), this, SLOT(deleteAllBreakpoints()));

    listview = new Q3ListView(0);
    pdpView->tabWidget->addTab(listview, QString());
    pdpView->tabWidget->setTabText(
      pdpView->tabWidget->indexOf(listview),
      i18n("Syntax Check")
    );
    enableDebuger(false);

    KConfigGroup cg(KGlobal::config(), "perl_syntax_check_plugin");
    m_sApacheLogFile = cg.readEntry("apache_error_log_file", m_sApacheLogFile);
    m_sPrefix        = cg.readEntry("source_code_prefix", m_sPrefix);
    m_sPerlCritcPath = cg.readEntry("perl_critic_path", m_sPerlCritcPath);
    m_sPdfReaderPath = cg.readEntry("pdf_reader_path", m_sPdfReaderPath);
    m_nSeverity      = cg.readEntry("serverity", m_nSeverity);
    m_sPbP           = cg.readEntry("pbppdf", m_sPbP);
    m_sArgs          = cg.readEntry("args", m_sArgs);
    m_sPath          = cg.readEntry("path", m_sPath);
    m_pEnv = QProcessEnvironment::systemEnvironment();
    KConfigGroup cgEnVars(KGlobal::config(), "perl_enVars");
    QStringList env = cgEnVars.keyList();
    for (int i = 0; i < env.size()  ;++i )
    {
        QString property =  env.at(i) ;
        QString value = cgEnVars.readEntry(property, "");
        QHash<QString, QString> hash;
        hash[property]=value;
        m_hVars.append( hash);
        m_pEnv.insert(property, value);
    }
    QAction *a = actionCollection()->addAction("perl_critic");
    a->setText(i18n("Perl::Critic"));
    a->setIcon( KIcon("perlCritic") );
    connect(a, SIGNAL(triggered()), this, SLOT(slotCritic()));

    QAction *b = actionCollection()->addAction("perl_check");
    b->setText(i18n("perl -c"));
    b->setIcon( KIcon("perlSyntaxcheck") );
    connect(b, SIGNAL(triggered()), this, SLOT(slotValidate()));

    QAction *c = actionCollection()->addAction("perl_apache");
    c->setText(i18n("Apache Error Log"));
    c->setIcon( KIcon("log") );
    connect(c, SIGNAL(triggered()), this, SLOT(apacheError()));   
 
    QAction *d = actionCollection()->addAction("trunlog");
    d->setText(i18n("Truncate Apache Error Log"));
    d->setIcon( KIcon("trash-empty") );
    connect(d, SIGNAL(triggered()), this, SLOT(trunlog()));   

    QAction *e = actionCollection()->addAction("apache_start");
    e->setText(i18n("Apache start"));
    e->setIcon( KIcon("media-playback-start") );
    connect(e, SIGNAL(triggered()), this, SLOT(start())); 

    QAction *f = actionCollection()->addAction("apache_stop");
    f->setText(i18n("Apache stop"));
    f->setIcon( KIcon("media-playback-stop") );
    connect(f, SIGNAL(triggered()), this, SLOT(stop()));

    QAction *g = actionCollection()->addAction("apache_restart");
    g->setText(i18n("Apache restart"));
    g->setIcon( KIcon("media-playlist-repeat") );
    connect(g, SIGNAL(triggered()), this, SLOT(restart()));     

    QAction *h = actionCollection()->addAction("apache_status");
    h->setText(i18n("Apache status"));
    h->setIcon( KIcon ("info") );
    connect(h, SIGNAL(triggered()), this, SLOT(status()));        
    
    QAction *k = actionCollection()->addAction("perl_debug_help");
    k->setText(i18n("Help"));
    k->setIcon( KIcon ("info") );
    connect(k, SIGNAL(triggered()), this, SLOT(help()));
    
    aStartX = actionCollection()->addAction("apache_startX");
    aStartX->setText(i18n("apache2ctl -X "));
    aStartX->setIcon( KIcon ("startX")  );
    aStartX->setToggleAction(true);
    connect(aStartX, SIGNAL(toggled(bool)), this, SLOT(startX(bool)));         
    
    aStartDebug = actionCollection()->addAction("apache_startDebug");
    aStartDebug->setText(i18n("apache2ctl -X -D PERLDB"));
    aStartDebug->setIcon(  KIcon ("mod_perlDb") );
    aStartDebug->setToggleAction(true);
    connect(aStartDebug, SIGNAL(toggled(bool)), this, SLOT(startPERLDB(bool)));

    setComponentData(KComponentData("kate"));
    setXMLFile("plugins/kateperlcheck/ui.rc");

    listview->setFocusPolicy(Qt::NoFocus);
    listview->addColumn(i18n("#"), 50);
    listview->addColumn(i18n("Line"), 50);
    listview->setColumnAlignment(1, Qt::AlignLeft);
    listview->addColumn(i18n("Column"), 50);
    listview->setColumnAlignment(2, Qt::AlignLeft);
    listview->addColumn(i18n("Message"), -1);
    listview->setAllColumnsShowFocus(true);
    listview->setResizeMode(Q3ListView::LastColumn);
    
    pdpView->tabWidget->setCurrentIndex( pdpView->tabWidget->indexOf(listview ) );
    connect(pdpView, SIGNAL(excute(QString)),this,SLOT(expr(QString)));

    connect(listview, SIGNAL(clicked(Q3ListViewItem*)), SLOT(slotClicked(Q3ListViewItem*)));
    connect(listview, SIGNAL(contextMenuRequested(Q3ListViewItem *, const QPoint &, int)), SLOT(contextMenu(Q3ListViewItem *, const QPoint &, int )));

    m_proc = new KProcess();
    connect( m_proc, SIGNAL(readyReadStandardError()), this,SLOT(readOutput()));
    connect( m_proc, SIGNAL(readyReadStandardOutput()),this,SLOT(readOutput()));
    m_proc->setOutputChannelMode(KProcess::MergedChannels);//standart output mit lesen.

    m_menu = new KActionMenu( i18n("Open sub"), this );
    actionCollection()->addAction( "menu_open_sub", m_menu );
    connect(m_menu->menu() ,SIGNAL(aboutToShow() ),this ,SLOT(openSub()) );

    connect( mainwin ,SIGNAL( viewCreated(KTextEditor::View*)),this,SLOT( viewCreated(KTextEditor::View*)));
    connect( mainwin ,SIGNAL( viewChanged()),this,SLOT( viewChanged()));    
    mainwin->guiFactory()->addClient(this);
}

/**
* \brief open kdehelpcenter
*  run follwing line to reindex the khelpcenter index.
* /usr/lib/kde4/libexec/khc_docbookdig.pl --indexdir=/usr/share/doc/kde/HTML/  --docpath=khelpcenter:kde_application_manuals --identifier=kde_application_manual --lang=en
* 
*/

void PluginKatePERLCheckView::help()
{
    KToolInvocation::invokeHelp("kate-perl-plugin", "kateperlcheck");
}

/**
* \brief openSub
*  right Click Menu in Editor Context Menu. 
*/

void PluginKatePERLCheckView::openSub()
{
  KateView *kateView = (KateView*)win->activeView();
  m_menu->menu()->clear();
  kDebug() << "Where";
  if( kateView->selection())
  {
    if(m_openProc)
      m_openProc->close();
    m_openProc = new KProcess();
    connect( m_openProc, SIGNAL(readyReadStandardError()), this,SLOT(findSubs()));
    connect( m_openProc, SIGNAL(readyReadStandardOutput()),this,SLOT(findSubs()));
    m_openProc->setOutputChannelMode(KProcess::MergedChannels);
    KUrl url = win->activeView()->document()->url();
    QString selectedText = kateView->selectionText();
    m_openProc->setShellCommand("findsub --file="+ url.path() +" --sub="+ selectedText);
    m_openProc->start();
  }
}

/**
* \brief findSubs
*  called by m_openProc (readyReadStandardOutput), add the openSub Menu to the Editor Context Menu.
* \see openSub
*/

void PluginKatePERLCheckView::findSubs()
{
    KUrl url = win->activeView()->document()->url();

    while(m_openProc->canReadLine())
    {
      
      QString line = m_openProc->readLine();
      QString file        = ""; 
      QString lineNumber  = "";
      QString name        = "";
      QRegExp rx("(.+)\\t(.+)\\t(.+)");
      if(rx.indexIn(line) != -1)
      {
        name        = rx.cap(1);
        file        = rx.cap(2);
        lineNumber  = rx.cap(3);
        QAction *action = new QAction( tr("Open sub"), this);
        action->setText(name);
        QStringList list;
        list << name << file <<lineNumber;
        QVariant data = QVariant(list);
        action->setData(data);
        connect(action, SIGNAL(triggered()), this, SLOT(gotoSub()));
        m_menu->addAction( action );
      }
    }
}

/**
* \brief gotoSub
*  Called by a QAction to open sub.
* \see findSubs 
*/

void PluginKatePERLCheckView::gotoSub()
{
  QStringList list=static_cast<QAction*>(sender())->data().toStringList();
  QString file = list.at(1);
  bool ok = true;
  uint line =  list.at(2).toUInt(&ok);
  KUrl url = file;
  if( ok )
  {
    KTextEditor::View *kv = win->openUrl(url);            
    if( ! kv )
        return;        
      win->activeView()->setCursorPosition(KTextEditor::Cursor (line-1, 1));
  }
}

/**
* \brief iconBorder debug sympol state changed
* This function is called whenever a breakpoint is set or unset.
* \param KTextEditor::Document*  unused
* \param KTextEditor::Mark mark pointer to the breakpoint
* \param KTextEditor::MarkInterface::MarkChangeAction change 
* \see KTextEditor::Document::markChanged()  [signal]
*/

void PluginKatePERLCheckView::markChanged( KTextEditor::Document* doc, KTextEditor::Mark mark, KTextEditor::MarkInterface::MarkChangeAction change){

  if( !m_bmarkChanged &&  mark.type & KTextEditor::MarkInterface::BreakpointActive)
  {
    
    if(change)
    {
      removeBreakPoint(mark.line,doc->url()); 
    }
    else
    {
      addBreakPoint(mark.line,doc->url());
      if(mark.type & KTextEditor::MarkInterface::Bookmark)
       ((KateView*)doc)->doc()->removeMark( mark.line,KTextEditor::MarkInterface::Bookmark);
    }
  }
  m_nMarkLine = mark.line;
}

/**
* \brief don't show mark Bookmark and Breakpoint at the same line.
* removes Bookmark Icon
*/

void PluginKatePERLCheckView::marksChanged(){
  KateView *kateView = (KateView*)win->activeView();
  uint mark = kateView->doc()->mark(m_nMarkLine);
  if(mark & KTextEditor::MarkInterface::Bookmark && mark & KTextEditor::MarkInterface::BreakpointActive)
    kateView->doc()->removeMark( m_nMarkLine,KTextEditor::MarkInterface::Bookmark);
}

/**
* \brief Destruktor
* remove the plugin from guiFactory
*/

PluginKatePERLCheckView::~PluginKatePERLCheckView()
{
    mainWindow()->guiFactory()->removeClient( this );
}

/**
* \brief add or remove (like toggle) a breakpoint at curent Line.
*/

void PluginKatePERLCheckView::addBreakPoint()
{
  KateView *kateView = (KateView*)win->activeView();
  uint mark = kateView->doc()->mark( kateView->cursorPosition().line() );
  
  if( mark == 0)
  {
    
    addBreakPoint( kateView->cursorPosition().line(),win->activeView()->document()->url());
    aBreakPoint->setChecked(true); 
  }
  else
  {
    removeBreakPoint( kateView->cursorPosition().line(),win->activeView()->document()->url());
    aBreakPoint->setChecked(false);
  }
}

/**
* \brief remove  breakpoint.
* \param uint line linenumber
*/
void PluginKatePERLCheckView::removeBreakPoint(uint line, KUrl url)
{
    KUrl cUrl = win->activeView()->document()->url();
    
    if(url != cUrl)
      return;
    
    QString proc = "B ";
    proc += QString::number(line+1);
    procWrite(proc.toLocal8Bit()+"\n" );

    KateView *kateView = (KateView*)win->activeView();

    for (int i = 0; i < m_aBreakpoints.size(); i++) 
    {
        QHash<KUrl, int> breakp = m_aBreakpoints.at(i);     
        QHash<KUrl, int>::const_iterator j = breakp.constBegin();
        
        if(line == (uint)j.value() && url == j.key()  )
            m_aBreakpoints.removeAt(i);
        
        uint mark = kateView->doc()->mark( line );
        m_bmarkChanged = true;
        
        if( mark & KTextEditor::MarkInterface::BreakpointActive )
        {
          kateView->doc()->removeMark( line,KTextEditor::MarkInterface::BreakpointActive );
        }
        m_bmarkChanged = false;
    }
}
/**
* \brief add breakpoint.
* \param uint line linenumber
*/
void PluginKatePERLCheckView::addBreakPoint(uint line,KUrl url)
{
    QString proc = "b ";
   
    if(m_bModPerl) 
      proc = "|b ";
    
    if(url != "")
    {
      proc += url.path();
      proc += ":";
    }
    proc += QString::number(line+1);
    procWrite(proc.toLocal8Bit()+"\n");
    QHash<KUrl, int> breakp;
    breakp[url]=line;
    m_aBreakpoints.append(breakp);
    QList<KTextEditor::View *> list= win->views();
    
    for (int i = 0; i < list.size(); ++i) 
    {
      KateDocument *doc = ((KateView*)list.at(i))->doc();
      
      if( doc->url() == url)
      {
          m_bmarkChanged = true;
          doc->addMark( line, KTextEditor::MarkInterface::BreakpointActive );
          m_bmarkChanged = false;
      }
    }
}

/**
* \brief read breakpoints from the Kprocess
*/
void PluginKatePERLCheckView::readBreakPoints(){
    KateDocument *doc =  ((KateView*)win->activeView())->doc();
    for (int i = 0; i < m_aBreakpoints.size(); ++i)
    {
     QHash<KUrl, int> breakp = m_aBreakpoints.at(i);
     QHash<KUrl, int>::const_iterator j = breakp.constBegin();
     int line = j.value();
     KUrl url = j.key();
     if( doc->url() == url)
     {
          m_bmarkChanged = true;
          doc->addMark( line, KTextEditor::MarkInterface::BreakpointActive );
          m_bmarkChanged = false;
      }
    }
}

/**
* \brief show tooltip for Variables.
* \param view
* \param newPosition 
*/

void PluginKatePERLCheckView::mousePositionChanged (KTextEditor::View *view, const KTextEditor::Cursor& newPosition){
  KateDocument* doc = ((KateView*)view)->doc();
  if(  newPosition.line() != -1 && ( m_bDebugProc || m_bModPerl ) )
  {
    QString word = doc->getWord( newPosition);
    QPoint p = view->mapToGlobal(view->pos());
    QRegExp rx("\\$|@|%");
    
    if(rx.indexIn(word) != -1 )
    {
      m_bGetVar = true;
      procWrite( ( m_bModPerl ? "|x " :"x " ) + word.toLocal8Bit()+"\n" );
      if(!m_Tip)
        m_Tip = new Tip();
        m_Tip->showTip(p + view->cursorToCoordinate(newPosition), m_sToolTip.trimmed() ) ;
    }
  }
  
}

/**
* \brief close the ToolTip onclick if isVisible
*/

void PluginKatePERLCheckView::cursorPositionChanged (
                                KTextEditor::View *view,
                                const KTextEditor::Cursor& newPosition
)
{
  if(m_Tip)
    m_Tip->close();
  KateView *kateView = (KateView*)view;
  KateDocument* doc = kateView->doc();
  QHash<int, KTextEditor::Mark*> m = doc->marks();
  aBreakPoint->setChecked(false);
  for (QHash<int, KTextEditor::Mark*>::const_iterator j = m.constBegin(); j != m.constEnd(); ++j)
  {
    if(j.value()->type &  KTextEditor::MarkInterface::BreakpointActive && j.value()->line == newPosition.line() )
       aBreakPoint->setChecked(true);
  }
}

/**
* \brief start or stop the debugger
* \param state 
*/

void PluginKatePERLCheckView::debug(bool state)
{
    if(!state)
    {
      cancel();
      return;
    }
    
    if(m_bModPerl)
      stopPERLDB();
    enableDebuger(true);
    aStartDebug->setDisabled(false);
    KateView *kateView = (KateView*)win->activeView();
    kateView->setMouseTrackingEnabled(true);
    disconnect(kateView,SIGNAL(mousePositionChanged (KTextEditor::View*, const KTextEditor::Cursor&)),this,SLOT(mousePositionChanged (KTextEditor::View*, const KTextEditor::Cursor&)));
    connect(
      kateView,
      SIGNAL(mousePositionChanged (KTextEditor::View*, const KTextEditor::Cursor&)),
      this,
      SLOT(mousePositionChanged (KTextEditor::View*, const KTextEditor::Cursor&))
    );
    
    disconnect(
      kateView,
      SIGNAL(cursorPositionChanged (KTextEditor::View *,
                                const KTextEditor::Cursor& )),
      this,
      SLOT(cursorPositionChanged (KTextEditor::View *,
                                const KTextEditor::Cursor& ))
    );        
    connect(
      kateView,
      SIGNAL(cursorPositionChanged (KTextEditor::View *,
                                const KTextEditor::Cursor& )),
      this,
      SLOT(cursorPositionChanged (KTextEditor::View *,
                                const KTextEditor::Cursor&))
    );    
    
    setIconBorder(kateView);
    createDebugProcess();

    KUrl url = win->activeView()->document()->url();
    m_DebugProc->setWorkingDirectory(url.directory());
   
    QString cmd = m_sPerl5+" "+m_sArgs+" "+url.path();
    QString argv = " "+pdpView->evalLine->currentText();
    if(argv !="")
      cmd += argv;
    pdpView->clear();
    pdpView->append(cmd);
    m_DebugProc->setShellCommand(cmd );
    m_DebugProc->start();
    rewriteBreakpoints();
    m_bDebugProc = true;
}

/**
* \brief create KProcess m_DebugProc
*/

void PluginKatePERLCheckView::createDebugProcess()
{
   m_DebugProc = new Perl();
   m_DebugProc->setProcessEnvironment(m_pEnv);  
   connect( m_DebugProc, SIGNAL(readLine(QString)), this,SLOT(readDebugOutput(QString)));
}

/**
* \brief read the debug output from kProcess.
*/

void PluginKatePERLCheckView::readDebugOutput(QString line)
{
    KUrl url = win->activeView()->document()->url();
    QString gFile;
    QString file        = "";
    QString lineNumber  ="";
    QRegExp rxPerl("(\\(.+\\))?\\((.+):(\\d+)\\)");
    if(rxPerl.indexIn(line) != -1)
    {
      file        = rxPerl.cap(2);
      lineNumber  = rxPerl.cap(3);
    }
    QRegExp rxRegistryPerl("\\[(.+):\\d+\\]:(\\d+)");
    if(rxRegistryPerl.indexIn(line) != -1)
    {
      file        = rxRegistryPerl.cap(1);
      lineNumber  = rxRegistryPerl.cap(2);
    }
    QRegExp rxModPerl("local $0 = '([^'+])'");
    if(rxModPerl.indexIn(line) != -1)
    {
      gFile        = rxModPerl.cap(1);
    }
    QRegExp rxLineModPerl("^(\\d+):");
    if(rxLineModPerl.indexIn(line) != -1)
    {
      lineNumber     = rxLineModPerl.cap(1);
      file = gFile;
    }
    if(file.length()>0 && lineNumber.length()>0 )
    {
      if(!QFile::exists(file)){
        if(QFile::exists(url.directory()+"/"+file))
          file = url.directory()+"/"+file;
        if(QFile::exists(m_DebugProc->workingDirectory()+"/"+file))
          file = m_DebugProc->workingDirectory()+"/"+file; 
      }
      if(!QFile::exists(file))
        return;
      KTextEditor::View *kv = win->openUrl(file);
      setIconBorder((KateView*)kv);
      readBreakPoints();
      if(kv)
        win->activeView()->setCursorPosition(KTextEditor::Cursor(lineNumber.toInt()-1, 1));
        pdpView->evalLine->setFocus();
    }
    if(m_bGetVar)
    {
        m_sToolTip = line;
    }else
      pdpView->append(line);
}

/**
* \brief slot which enables the Iconborder.
* 
*/

void PluginKatePERLCheckView::viewCreated(KTextEditor::View* v)
{
  setIconBorder((KateView*)v);
  connect(
  (KateView*)v,
  SIGNAL(cursorPositionChanged (KTextEditor::View *,
                            const KTextEditor::Cursor& )),
  this,
  SLOT(cursorPositionChanged (KTextEditor::View *,
                            const KTextEditor::Cursor&))
  );
}

/**
* \brief slot which enables the Iconborder.
* 
*/

void PluginKatePERLCheckView::viewChanged ()
{
  KateView* kv = (KateView*)win->activeView();
  disconnect(kv,SIGNAL(mousePositionChanged (KTextEditor::View*, const KTextEditor::Cursor&)),this,SLOT(mousePositionChanged (KTextEditor::View*, const KTextEditor::Cursor&)));
  connect(
    kv,
    SIGNAL(mousePositionChanged (KTextEditor::View*, const KTextEditor::Cursor&)),
    this,
    SLOT(mousePositionChanged (KTextEditor::View*, const KTextEditor::Cursor&))
  );
  
  disconnect(
  kv,
  SIGNAL(cursorPositionChanged (KTextEditor::View *,
                            const KTextEditor::Cursor& )),
  this,
  SLOT(cursorPositionChanged (KTextEditor::View *,
                            const KTextEditor::Cursor& ))
  );        
  connect(
  kv,
  SIGNAL(cursorPositionChanged (KTextEditor::View *,
                            const KTextEditor::Cursor& )),
  this,
  SLOT(cursorPositionChanged (KTextEditor::View *,
                            const KTextEditor::Cursor& ))
  );  
    
  if(kv)
    setIconBorder(kv);
}

/**
* \brief create and connect the Iconborder to given KateView
* \param view 
*/

//need KateViewConfig::global()
#include <part/utils/kateconfig.h>

void PluginKatePERLCheckView::setIconBorder(KateView* kateView)
{
   kateView->setIconBorder(true);
   disconnect(kateView->doc(),SIGNAL(markChanged( KTextEditor::Document*,KTextEditor::Mark,KTextEditor::MarkInterface::MarkChangeAction)),this,SLOT(markChanged( KTextEditor::Document*,KTextEditor::Mark,KTextEditor::MarkInterface::MarkChangeAction))); 
   disconnect( kateView->doc(), SIGNAL(marksChanged(KTextEditor::Document*)), this, SLOT(marksChanged()));
   connect(
      kateView->doc(),
      SIGNAL(
        markChanged( 
          KTextEditor::Document*,
          KTextEditor::Mark,
          KTextEditor::MarkInterface::MarkChangeAction
        )
      ),
      this,
      SLOT(
        markChanged( 
          KTextEditor::Document*,
          KTextEditor::Mark,
          KTextEditor::MarkInterface::MarkChangeAction
        )
      )
    );

  connect( 
    kateView->doc(), SIGNAL(marksChanged(KTextEditor::Document*)), this, SLOT(marksChanged()));   
    kateView->setIconBorder(true);
    ((KateDocument*)win->activeView()->document())->setMarkPixmap(
      KTextEditor::MarkInterface::BreakpointActive,
      KIcon("script-error").pixmap(16, 16)
    );
    kateView->doc()->setMarkDescription( KTextEditor::MarkInterface::BreakpointActive, "&Breakpoint" );
    kateView->doc()->setEditableMarks( KTextEditor::MarkInterface::Bookmark +KTextEditor::MarkInterface::BreakpointActive );
    KateViewConfig::global()->setDefaultMarkType (KTextEditor::MarkInterface::BreakpointActive);
}

/**
* \brief reset variables after readOutput
* within readOutput there are is m_bGetVar to skip printing messages to output window. 
* and m_bReadBreakPoints that tells m_bReadBreakPoints that breakpoints should read from STDOUT
*/

void PluginKatePERLCheckView::resetVars()
{
  m_bGetVar = false;

}

/**
* \brief enable or disable KActions while debugging.
* \param enable 
*/

void PluginKatePERLCheckView::enableDebuger(bool enable)
{
  aReStart->setDisabled(!enable); 
  aStep->setDisabled(!enable);
  aReturnFS->setDisabled(!enable); 
  aContinue->setDisabled(!enable);
  aStepOver->setDisabled(!enable);
  aDelAll->setDisabled(!enable);
  pdpView->tabWidget->setCurrentIndex(pdpView->tabWidget->indexOf(pdpView->Debug));
  if(dock)
  {
    dock->showNormal();
    win->showToolView (dock);
  }   
}

/**
* \brief restart Debugger.
*/

void PluginKatePERLCheckView::debugRestart()
{
  resetVars();
  procWrite("R\n");
  rewriteBreakpoints();
  pdpView->clear();
}

/**
* \brief write an command to the Debugger process.
*/

qint64 PluginKatePERLCheckView::procWrite(const char *data)
{
  if(m_DebugProc)
  {
    qint64 nReturnCode =  m_DebugProc->write(data);
    return nReturnCode;
  }
  else
    return 0;
}

/**
* \brief get breakpoints from the IconBorder and write them to the Debugger process.
*/

void PluginKatePERLCheckView::rewriteBreakpoints()
{
  QList<KTextEditor::View *> list= win->views();
  
  for (int i = 0; i < list.size(); ++i)
  {
    QHash<int, KTextEditor::Mark*> m = ((KateView*)list.at(i))->doc()->marks();
    for (QHash<int, KTextEditor::Mark*>::const_iterator j = m.constBegin(); j != m.constEnd(); ++j)
    {
      if(j.value()->type &  KTextEditor::MarkInterface::BreakpointActive )
        addBreakPoint(j.value()->line,((KateView*)list.at(i))->doc()->url());
    }
  }
}

/**
* \brief abort debugging.
*/

void PluginKatePERLCheckView::cancel()
{
  aStartDebug->setDisabled(false);
  enableDebuger(false);
  resetVars();
  procWrite("q\n" );
  m_DebugProc->close();
  disconnect( m_DebugProc, SIGNAL(readyReadStandardError()), this,SLOT(readDebugOutput()));
  disconnect( m_DebugProc, SIGNAL(readyReadStandardOutput()),this,SLOT(readDebugOutput()));
  m_bDebugProc = false;
}

/**
* \brief write expr to Debugger process.
* \param expr 
*/

void PluginKatePERLCheckView::expr(QString expr)
{
  if(expr == "R")
  {
    debugRestart();
  }
  if(expr == "start")
  {
    pdpView->evalLine->setCurrentText("");
    debug(true);
    aDebug->toggle();
  }
  else if(expr == "B")
  {
    deleteAllBreakpoints();
  }
  else if(expr.contains(QRegExp("^[b|B].*")))
  {
    QRegExp rx("^([b|B]) ((.*):)?(\\d+)");
    
    if (rx.indexIn(expr) != -1)
    {
      QString action = rx.cap(1);
      QString file   = rx.cap(3);
      QString line   =  rx.cap(4);
      int nLine     = line.toUInt();
      
      if( action == "b")
        addBreakPoint(nLine+1,KUrl(file));
      else if(action =="B")
        removeBreakPoint(nLine+1, win->activeView()->document()->url());
    }
  }
  else if(m_bModPerl && !expr.contains(QRegExp("(R|s|n|r|c)")))//run not through your pager (LineInfo) 
  {
    expr = "|"+expr;
    resetVars();
    procWrite(expr.toLocal8Bit() + "\n" );
  }
  else
  {
    resetVars();
    procWrite(expr.toLocal8Bit() + "\n" );
  }

}

/**
* \brief Debug ( s ) single step
*/

void PluginKatePERLCheckView::singleStep()
{
  resetVars();
  procWrite("s\n");
}

/**
* \brief Debug ( n ) StepOver
*/

void PluginKatePERLCheckView::stepOver()
{
  resetVars(); 
  procWrite("n\n");

}
/**
* \brief Debug ( r ) return from sub
*/

void PluginKatePERLCheckView::returnFromSub()
{
  resetVars();
  procWrite("r\n");
}

/**
* \brief Debug ( c ) continue until position
*/

void PluginKatePERLCheckView::continueUntilPosition()
{
  resetVars();
  procWrite("c\n");
}

/**
* \brief Debug ( B ) delete all Breakpoints
*/

void PluginKatePERLCheckView::deleteAllBreakpoints()
{
  resetVars();
  procWrite( m_bModPerl ? "|B\n" :"B\n" );
  QList<KTextEditor::View *> list= win->views();
  for (int i = 0; i < list.size(); ++i) {
    QHash<int, KTextEditor::Mark*> m = ((KateView*)list.at(i))->doc()->marks();
    for (QHash<int, KTextEditor::Mark*>::const_iterator j = m.constBegin(); j != m.constEnd(); ++j)
      ((KateView*)list.at(i))->doc()->removeMark( j.value()->line, KTextEditor::MarkInterface::BreakpointActive );
  }
}

/**
* \brief creates the context Menu for Output listview
*/

void PluginKatePERLCheckView::contextMenu(Q3ListViewItem * item,const QPoint & point, int col ){
    Q_UNUSED(col);
    if(item){
      Q3PopupMenu *menu = new Q3PopupMenu();
      if(m_bApache)
      {
          QAction* openAct = new QAction(QIcon(":/images/open.png"), tr("Open with prefix"), this);
          openAct->setStatusTip(tr("Open File within sourcecode, replace /var/www/ with refix ..."));
          disconnect(openAct, SIGNAL(triggered()), this, SLOT(open()));  
          connect(openAct, SIGNAL(triggered()), this, SLOT(open()));  
          menu->addAction(openAct);
      }
      if(m_bCritic)
      {
          QAction * openPdfAction = new QAction(QIcon(":/images/open.png"), tr("&Open pbp"), this);
          openPdfAction->setStatusTip(tr("Open Berl best Practices pdf for help"));
          disconnect(openPdfAction, SIGNAL( triggered() ), this, SLOT( openPdf() ));  
          connect(openPdfAction, SIGNAL( triggered() ), this, SLOT( openPdf() ));  
          menu->addAction(openPdfAction);
      }
      if( m_bApache || m_bCritic)
        menu->exec(point);
    }
}

/**
* \brief open with "prefix". Used to open an file outside the given Document Root.
*/

void PluginKatePERLCheckView::open(){
    Q3ListViewItem * item  = listview->currentItem();
    if(item){
      QString file = item->text(1);
      QRegExp rx(m_sPath+"(.+[\\.pm|\\.pl])$");
      if (rx.indexIn(file) != -1) {
          file        = rx.cap(1);
      }
      bool ok = true;
      uint line = item->text(2).toUInt(&ok);
      QString dir = m_sPrefix;
      dir.append("/");
      KUrl url = dir+= file;
      if( ok ) {
        KTextEditor::View *kv = win->openUrl(url);            
        if( ! kv )
            return;        
          win->activeView()->setCursorPosition(KTextEditor::Cursor (line-1, 1));
      }
    }
}


/**
* \brief open with "prefix". Used to open an file outside the given Document Root.
*/

void PluginKatePERLCheckView::readOutput()
{
    QApplication::restoreOverrideCursor();
    while(m_proc->canReadLine())
    {
        QString line = m_proc->readLine();
        if(m_bCritic)
        {
            insertItem(QRegExp("^(.+)at line.(\\d+).*column.(\\d+)(.*)$"),line);
        }
        else
        {
            insertItem(QRegExp("^(.+) line.(\\d+),?(.*)$"),line);
        }
    }
    if(list_count == 0)
    {
      new Q3ListViewItem(
          listview,
          QString::number(list_count).rightJustified(4,' '), 
          "",
          "",
          i18n("No error")
      );
    }
}

/**
* \brief open current "critic" or "error" file, line within text the Editor.
*/

void PluginKatePERLCheckView::slotClicked(Q3ListViewItem *item)
{
    if( item ) {
      if(m_bApache)
      {
        QString file = item->text(1);
        bool ok = true;
        uint line = item->text(2).toUInt(&ok);
        KUrl url = file;
        if( ok ) {
          KTextEditor::View *kv = win->openUrl(url);            
          if( ! kv )
              return;        
            win->activeView()->setCursorPosition(KTextEditor::Cursor (line-1, 1));
        }
      } 
      else 
      {
        bool ok = true;
        uint line = item->text(1).toUInt(&ok);
        bool ok2 = true;
        uint column = m_bCritic ? item->text(2).toUInt(&ok2) : 0;
          if( ok && ok2 )
          {
            KTextEditor::View *kv = win->activeView();
            if( ! kv )
              return;
            kv->setCursorPosition(KTextEditor::Cursor (line-1, column));
          }
      }
    }
}


/**
* \brief validate [perl -c]  current Document.
*/

bool PluginKatePERLCheckView::slotValidate()
{
  m_bCritic = false;
  m_bApache = false;
  listview->setColumnText(1,i18n("Line"));
  listview->setColumnWidth(1,50);
  listview->setColumnText(2,i18n("Near"));
  listview->setColumnText(3,i18n("Message"));
  KUrl url = win->activeView()->document()->url();
  startProc("perl -c "+url.path());  
  return true;
}

/**
* \brief validate [perlcritic ]  current Document.
*/

bool PluginKatePERLCheckView::slotCritic()
{
  m_bCritic = true;
  m_bApache = false;
  listview->setColumnText(1,i18n("Line"));
  listview->setColumnWidth(1,50);
  listview->setColumnText(2,i18n("Column"));
  listview->setColumnText(3,i18n("Message"));
  KUrl url = win->activeView()->document()->url();
  startProc(m_sPerlCritcPath+" -"+ QString::number(m_nSeverity)+" "+url.path());
  return true;
}

/**
* \brief validate [Apache error Log]  current Document.
*/

bool PluginKatePERLCheckView::apacheError()
{
  m_bCritic = false;
  m_bApache = true;
  listview->setColumnText(1,i18n("File"));
  listview->setColumnWidth(1,250);
  listview->setColumnText(2,i18n("Line"));
  listview->setColumnText(3,i18n("Message"));
  listview->clear();
  list_count = 0;
  win->showToolView (dock);
  m_proc->close();
  pdpView->tabWidget->setCurrentIndex(pdpView->tabWidget->indexOf(listview ) );
  QFile file(m_sApacheLogFile);
  if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
      return false;
  QTextStream in(&file);
  while (!in.atEnd())
  {
    QString line = in.readLine();
    insertItem(QRegExp("AH[^\\]]+\\](.+) at (.+) line (\\d+)(.*)$"),line);
    insertItem(QRegExp(":(.+) at (\\S+) line (\\d+),?(.*)$"),line);
  }
  return true;
}

/**
* \brief insert a Listview Item in the "Error Log window " 
* \param rx regular expression match msg lineumber column
* \param line line matched.
*/

bool PluginKatePERLCheckView::insertItem(QRegExp rx, QString line)
{
  if (rx.indexIn(line) != -1) 
  {
    QString msg        = rx.cap(1);
    QString linenumber = rx.cap(2);
    QString near       = rx.cap(3);
    if(m_bCritic)
        msg.append(rx.cap(4));
        list_count++;
        Q3ListViewItem *item = new Q3ListViewItem(
          listview,
          QString::number(list_count).rightJustified(4,' '), 
          linenumber.rightJustified(4,' '),
          near,
          msg
        );
    item->setMultiLinesEnabled(true);
    item->setRenameEnabled (1,true);
    item->setRenameEnabled (2,true);
    item->setRenameEnabled (3,true);
    return true;
  }
  return false;
}

/**
* \brief start validate process
* \param cmd command
*/

void PluginKatePERLCheckView::startProc(QString cmd)
{
  pdpView->tabWidget->setCurrentIndex(pdpView->tabWidget->indexOf(listview ) );
  
  listview->clear(); 
  list_count = 0;
  dock->showNormal();
  win->showToolView (dock);
  m_proc->close();
  KUrl url = win->activeView()->document()->url();
  m_proc->setWorkingDirectory(url.directory());
  m_proc->setShellCommand(cmd );
  m_proc->start();
//   if(url.path().length() > 0)
   QApplication::restoreOverrideCursor();
    QApplication::setOverrideCursor(Qt::WaitCursor);
//   else
//     
}

/**
* \brief truncate the Apache error Log.
*/

void PluginKatePERLCheckView::trunlog()
{
  KProcess* proc = new KProcess();
  proc->setShellCommand("truncate --size 0 /var/log/apache2/error.log " );
  proc->start();
}

/**
* \brief apache2ctl start
*/
void PluginKatePERLCheckView::start()
{
  KProcess* proc = new KProcess();
  proc->setShellCommand("kdesudo apache2ctl start && message=`apache2ctl status | grep uptime` && notify-send \"$message\"");
  proc->start();
}

/**
* \brief apache2ctl stop
*/

void PluginKatePERLCheckView::stop()
{
  KProcess* proc = new KProcess();
  proc->setShellCommand("kdesudo apache2ctl stop && notify-send Apache stoped" );
  proc->start();
}

/**
* \brief apache2ctl restart
*/

void PluginKatePERLCheckView::restart()
{
  KProcess* proc = new KProcess();
  proc->setShellCommand("kdesudo apache2ctl restart && message=`apache2ctl status | grep uptime` && notify-send \"$message\"" );
  proc->start();
}

/**
* \brief apache2ctl status
*/

void PluginKatePERLCheckView::status()
{
  KProcess* proc = new KProcess();
  proc->setShellCommand("message=`apache2ctl status | grep uptime` && notify-send \"$message\"" );
  proc->start();
}

/**
* \brief apache2ctl -X
*/

void PluginKatePERLCheckView::startX(bool state)
{
  if(!state)
  {
    stopX();
    return;
  }
  m_procX = new KProcess();
  m_procX->setShellCommand("kdesudo \"apache2ctl -X\"");
  m_procX->start();
}

/**
* \brief stop apache2ctl -X
*/

void PluginKatePERLCheckView::stopX()
{
   m_procX->close();
}

/**
* \brief debug Apache you need to install Apache::DB from cpan to debug mod_perl.
* And add the following lines to your Apache config.
*    <IfDefine PERLDB>
*    <Perl>
*    use APR::Pool ();
*    use Apache::DB ();
*    Apache::DB->init;
*    </Perl>
*     #location which should be debugged /cgi-bin/
*     <Location />
*     PerlFixupHandler Apache::DB
*    </Location>
*    </IfDefine>
* \şee https://perl.apache.org/docs/1.0/guide/debug.html
*/

void PluginKatePERLCheckView::startPERLDB(bool state)
{
  if(!state)
  {
    stopPERLDB();
    return;
  }
  aDebug->setDisabled(true);
  enableDebuger(true);
  m_bModPerl = true;
  createDebugProcess();
  m_DebugProc->setShellCommand("pkexec --user root apache2ctl -X -D PERLDB");
  m_DebugProc->start();
  m_DebugProc->write("o LineInfo=/dev/stdout\n");

}

/**
* \brief stop apache2ctl -X
*/

void PluginKatePERLCheckView::stopPERLDB()
{
    aDebug->setDisabled(false);
    stop();
    cancel();
    m_bModPerl = false;
}

/**
* \brief open perl best practices pdf.
*/

void PluginKatePERLCheckView::openPdf()
{
  Q3ListViewItem * item  = listview->currentItem();
  
  if(item)
  {
    QString page = "0";
    QString txt = item->text(3);
    QRegExp rx("pages? (\\d+)");
    
    if(rx.indexIn( txt ) != -1)
    {
        page = rx.cap(1);
    }
    KProcess* proc = new KProcess();
    proc->setShellCommand( m_sPdfReaderPath+" -p "+page+  " "+m_sPbP );
    proc->start();
  }
}

/**
* \brief save config to disk.
*/

void PluginKatePERLCheckView::saveSettings(
    QString errorPath,
    QString prefix,
    QString perlCriticPath,    
    QString pdfReaderPath,
    QString pbpPdf,
    uint serverity,
    QString perl5,
    QString args,
    QString path
    )
{
    KConfigGroup cg(KGlobal::config(), "perl_syntax_check_plugin");
    cg.writeEntry("apache_error_log_file", errorPath);
    cg.writeEntry("source_code_prefix", prefix);
    cg.writeEntry("perl_critic_path", perlCriticPath);
    cg.writeEntry("pdf_reader_path", pdfReaderPath);
    cg.writeEntry("serverity", serverity);
    cg.writeEntry("pbppdf", pbpPdf);
    cg.writeEntry("perl5", perl5);
    cg.writeEntry("args", args);
    cg.writeEntry("path", path);
    KConfigGroup cgEnVars(KGlobal::config(), "perl_enVars");
    cgEnVars.deleteGroup();
    for (int i = 0; i < m_hVars.size(); i++)
    { 
        QHash<QString, QString> hash =  m_hVars.at(i);     
        QHash<QString, QString>::const_iterator j = hash.constBegin();
        cgEnVars.writeEntry(j.key(), j.key());
    }

    m_sApacheLogFile = errorPath;
    m_sPrefix        = prefix;
    m_sPerlCritcPath = perlCriticPath;
    m_sPdfReaderPath = pdfReaderPath;
    m_sPbP           = pbpPdf;
    m_nSeverity      = serverity;
    m_sPerl5         = perl5;
    m_sArgs          = args;
    m_sPath          = path;
}

/**
* \brief pure virtual methods must be implemnted or cast will failed.
*/

PluginKatePERLCheck::PluginKatePERLCheck( QObject* parent, const QStringList& ) : Kate::Plugin ( (Kate::Application *)parent ), Kate::PluginConfigPageInterface()
{
  Q_UNUSED(parent);
}
/**
* \brief pure virtual methods must be implemnted or cast will failed.
*/
PluginKatePERLCheck::~PluginKatePERLCheck(){}

Kate::PluginConfigPage* PluginKatePERLCheck::configPage (uint number, QWidget *parent, const char *name)
{
  Q_UNUSED(name);
  if (number != 0)
    return 0;
  sDialog = new SettingsDialog(parent,m_view);
  connect( sDialog, SIGNAL(saveSettings(QString,QString, QString,QString,QString,uint,QString,QString,QString)), m_view, SLOT(saveSettings(QString,QString, QString,QString,QString,uint,QString,QString,QString)));
  return sDialog;
}
/**
* \brief pure virtual methods must be implemnted or cast will failed.
*/
QString PluginKatePERLCheck::configPageName (uint number) const{
  Q_UNUSED(number);
  return "Perl";
}
/**
* \brief return the "title of the Config Page.
*/
QString PluginKatePERLCheck::configPageFullName (uint number) const
{
  Q_UNUSED(number);
  return "Perl plugin Config Page";
}
/**
* \brief return the Icon for the Config Page.
*/
KIcon PluginKatePERLCheck::configPageIcon (uint number) const 
{
  Q_UNUSED(number);
  return KIcon("application-x-perl");
}

Kate::PluginView *PluginKatePERLCheck::createView(Kate::MainWindow *mainWindow)
{
    m_view =  new PluginKatePERLCheckView(mainWindow,sDialog);
    return m_view;
}

/**
* \brief create the Settings Dialog from settings.ui 
*/

SettingsDialog::SettingsDialog(
    QWidget *parent,
    PluginKatePERLCheckView* view
  ) : Kate::PluginConfigPage( parent )
{
  m_view = view;
  setupUi(this);

  Prefix->setText(m_view->m_sPrefix);
  Path->setText(m_view->m_sPath);
  ErrorPath->setText(m_view->m_sApacheLogFile);
  perlCritic->setText(m_view->m_sPerlCritcPath);
  pdfReader->setText(m_view->m_sPdfReaderPath);
  severity->setValue(m_view->m_nSeverity);
  pbpPdf->setText(m_view->m_sPbP);
  perl5->setText(m_view->m_sPerl5);
  args->setText(m_view->m_sArgs);
  for (int i = 0; i < m_view->m_hVars.size(); i++) 
  {
      QHash<QString, QString> hash =  m_view->m_hVars.at(i);     
      QHash<QString, QString>::const_iterator j = hash.constBegin();
      vars->insertRow(i);
      QTableWidgetItem *property = new QTableWidgetItem(j.key() );
      vars->setItem(i, 0, property);
      QTableWidgetItem *value = new QTableWidgetItem(j.value() );
      vars->setItem(i, 1, value);
  }
  connect(vars,SIGNAL(customContextMenuRequested ( const QPoint)),this,SLOT(customContextMenuRequested ( const QPoint) ));    
}


/**
* \brief QMenu for the  Environment List
*/

void SettingsDialog::customContextMenuRequested ( const QPoint & pos ) 
{
    QMenu *menu = new QMenu();
    QAction* removeAction = new QAction(  tr("Remove Var"), this);
    connect(removeAction, SIGNAL(triggered()), this, SLOT(removeVar()));  
    menu->addAction(removeAction); 
    QAction* addAction = new QAction( tr("Add Var"), this);
    connect(addAction, SIGNAL(triggered()), this, SLOT(addVar()));  
    menu->addAction(addAction);
    QPoint p = vars->mapToGlobal(pos);
    menu->exec(p);
}

/**
* \brief delete a row from the  Environment List
*/

void SettingsDialog::removeVar ()
{
vars->removeRow( vars->currentRow() ); 
}

/**
* \brief add a row to Environment List
*/

void SettingsDialog::addVar ()
{
vars->insertRow(vars->currentRow()+1);
}
/**
* \brief pure virtual methods must be implemnted or cast will failed.
*/
SettingsDialog::~SettingsDialog (){}

/**
* \brief emit the SIGNAL saveSettings
* şee saveSettings()
*/

void SettingsDialog::apply()
{
  m_view->m_hVars.clear ();
  m_view->m_pEnv.clear();
  for (int i = 0; i < vars->rowCount ()  ;++i )
  {
    QTableWidgetItem * itemKey = vars->item(i,0);
    QTableWidgetItem * itemValue = vars->item(i,1);
    if(itemKey && itemValue )
    {
      QString property = itemKey->text();
      QString value = itemValue->text();
      QHash<QString, QString> hash;
      hash[property]=value;
      m_view->m_hVars.append( hash);
      m_view->m_pEnv.insert(property, value);
    }
  }
  emit saveSettings(
      ErrorPath->text(),
      Prefix->text(),
      perlCritic->text(),
      pdfReader->text(),
      pbpPdf->text(),
      severity->value(),
      perl5->text(),
      args->text(),
      Path->text()
  );
}

/**
* \brief pure virtual methods must be implemnted or cast will fail.
*/

void SettingsDialog::reset(){}

/**
* \brief pure virtual methods must be implemnted or cast will fail.
*/

void SettingsDialog::defaults(){}

