/*Author: dirk.lze@gmail.com
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#ifndef PLUGIN_KATEPERLCHECK_H
#define PLUGIN_KATEPERLCHECK_H

#include <q3listview.h>
#include <qstring.h>
#include <kprocess.h>
//Added by qt3to4:
#include <Q3PtrList>
#include <QObject>
#include <kate/plugin.h>
#include <kate/pluginconfigpageinterface.h>
#include <kate/application.h>
#include <kate/documentmanager.h>
#include <kate/mainwindow.h>
#include <ktexteditor/view.h>
#include <ktexteditor/document.h>
#include <k3dockwidget.h>
#include <kiconloader.h>
#include <kateview.h>

#include "ui_settings.h"
#include "qpdbwidget.h"
#include <KActionMenu>

#include <QProcess>
/**
 * \mainpage Kate Perl Plug-In. Perl syntax check (perl -c ), Perl::Critic and and Debugger plugin.
 *
 *
 * \section plugin_install Install
 * 
 * build within kate's source code.
 * Copy the folder containing this file to addons/kate.
 * Add following line to addons/kate/CMakeLists.txt
 * 
 * add_subdirectory( perlcheck )
 * 
 * type :
 *  cmake CMakeLists.txt
 *  make
 *  make install
 *
 * See also: http://kate-editor.org/get-it/  
 * 
 * \section plugin_khelpcenter khelpcenter
 * 
 * To rebuild khelcenter index you need to install htdig and type:
 * /usr/lib/kde4/libexec/khc_docbookdig.pl --indexdir=/usr/share/doc/kde/HTML/  --docpath=khelpcenter:kde_application_manuals --identifier=kde_application_manual --lang=de
 * Per default the .desktop file will be installed at /usr/share/kde4/services/.
 * To get into the "Index" of kdehelpcenter you need to move this file to /usr/share/applications/kde4/.
*/
class Tip;

class Tip : public QWidget
{
  Q_OBJECT
public:
    Tip( QWidget* parent = 0);
    void showTip(QPoint p,QString str);
public slots:
    void hide ();
private:
   QTextBrowser* m_Tip;
   bool m_bLeave;
protected:
  virtual void leaveEvent ( QEvent * event ) ;
};

class Perl : public QObject
{
  Q_OBJECT

public:
    Perl( QObject* parent = 0);
    qint64 write(const char *data);
    void close();
    QString workingDirectory(){ return m_Perl->workingDirectory(); }
    void setShellCommand(const QString &cmd){m_Perl->setShellCommand(cmd);}
    void setWorkingDirectory(const QString &dir){m_Perl->setWorkingDirectory(dir); }
    void setProcessEnvironment(const QProcessEnvironment & environment){m_Perl->setProcessEnvironment(environment);}
public slots:
    void start();
    void readOutput();
signals:
     void readLine(QString);

private :
  KProcess *m_Perl = 0;
  QString output;

};

class SettingsDialog;

class PluginKatePERLCheckView : public Kate::PluginView, public KXMLGUIClient
{
    Q_OBJECT

public:
    PluginKatePERLCheckView(Kate::MainWindow *mainwin,SettingsDialog* sdialog);
    virtual ~PluginKatePERLCheckView();
    Kate::MainWindow *win;
    QWidget *dock;
    QString m_sApacheLogFile = "/var/log/apache2/error.log";
    QString m_sPath          = "/var/www/cgi-bin/";
    QString m_sPrefix        = "/another/path/cgi-bin/";
    QString m_sPerlCritcPath = "/usr/local/bin/perlcritic";
    QString m_sPdfReaderPath = "/usr/bin/atril";
    uint m_nSeverity         = 5;
    QString m_sPbP           = "/perl_best_practice_pdf";
    QString m_sPerl5         = "/usr/bin/perl";
    QString m_sArgs          = "-d";

    QList<QHash<QString,QString>> m_hVars;
    QProcessEnvironment m_pEnv;
public slots:
    void readDebugOutput(QString line);
    void debug(bool);
    void singleStep();
    void cancel();
    void stepOver();
    void returnFromSub();
    void continueUntilPosition();
    void deleteAllBreakpoints();
    void expr(QString expr);
    bool slotValidate();
    bool slotCritic();
    bool apacheError();
    void slotClicked(Q3ListViewItem*item);
    void readOutput();
    void contextMenu(Q3ListViewItem*,const QPoint &, int col );
    void addBreakPoint();
    void addBreakPoint(uint line, KUrl url = KUrl(""));
    void removeBreakPoint(uint line , KUrl url = KUrl(""));
    void readBreakPoints();
    void markChanged( KTextEditor::Document*, KTextEditor::Mark, KTextEditor::MarkInterface::MarkChangeAction );
    void marksChanged();
    void help();
    void openSub();
    void findSubs();
    void gotoSub();
    void mousePositionChanged (KTextEditor::View*, const KTextEditor::Cursor&);
    void cursorPositionChanged (KTextEditor::View *view, const KTextEditor::Cursor& newPosition);
    void open();
    bool insertItem(QRegExp,QString);
    void trunlog();
    void start();
    void stop();
    void restart();
    void status();
    void openPdf();
    void debugRestart();
    void enableDebuger(bool);
    void saveSettings(QString, QString,QString,QString,QString, uint,QString,QString,QString);
    void viewCreated(KTextEditor::View*);
    void viewChanged();
    void startPERLDB(bool);
    void startX(bool);
    void stopPERLDB();
    void stopX();
private:
    void resetVars();
    void setIconBorder(KateView*);
    void rewriteBreakpoints();
    void createDebugProcess();
    qint64 procWrite(const char *data);
    KParts::ReadOnlyPart *part;
    int list_count = 0;
    int m_nMarkLine = 0;
    KProcess *m_proc;
    Perl *m_DebugProc = 0;
    KProcess *m_openProc = 0;
    KProcess *m_procX = 0;
    Q3ListView *listview;
    QPdbWidget *pdpView;
    KActionMenu* m_menu ;
    QList<QHash<KUrl,int>> m_aBreakpoints;
    bool m_bCritic;
    bool m_bApache;
    bool m_bmarkChanged = false;
    bool m_bReadBreakPoints = false;
    void startProc(QString cmd);
    bool m_bGetVar;
    bool m_bModPerl = false;
    bool m_bDebugProc = false;
    QString m_sToolTip    ="";
    QString m_sOldTooltip = "none";
    QString m_sOldToolTip;
    QAction* aDebug;
    QAction* aReStart; 
    QAction* aStep;
    QAction* aReturnFS; 
    QAction* aContinue;
    QAction* aBreakPoint;
    QAction* aStepOver;
    QAction* aDelAll;
    QAction* aStartDebug;
    QAction* aStartX;
    QAction* m_aContextMenuBreakpoint = 0;
    Tip* m_Tip = 0;
};



class PluginKatePERLCheck : public Kate::Plugin, public Kate::PluginConfigPageInterface
{
  Q_OBJECT
  Q_INTERFACES(Kate::PluginConfigPageInterface)

public:
    explicit PluginKatePERLCheck( QObject* parent = 0,const QStringList& = QStringList());
    virtual ~PluginKatePERLCheck();
    Kate::PluginView *createView(Kate::MainWindow *mainWindow);
    SettingsDialog* sDialog;
    virtual Kate::PluginConfigPage *configPage (uint number = 0, QWidget *parent = 0, const char *name = 0);
    virtual uint configPages () const {return 1;};
    virtual QString configPageName(uint number = 0) const;
    virtual QString configPageFullName(uint number = 0) const;
    virtual KIcon configPageIcon(uint number = 0) const;
    PluginKatePERLCheckView* m_view;
};

class SettingsDialog : public Kate::PluginConfigPage, private Ui::SettingsDialog
{
     Q_OBJECT
public:
     SettingsDialog(QWidget *parent = 0, PluginKatePERLCheckView*  = 0);
     virtual ~SettingsDialog ();
     void setPrefix(QString string){Prefix->setText(string);};
     void setApacheLogFile(QString string){ErrorPath->setText(string);};
     void setPerlCriticPath(QString string){perlCritic->setText(string);};
     void setPdfReader(QString string){pdfReader->setText(string);};
     void setSeverity(uint num){severity->setValue(num);};
     void setPbpPdf(QString value){pbpPdf->setText(value);};     
     virtual void apply();
     virtual void reset();
     virtual void defaults();
     PluginKatePERLCheckView* m_view;
public slots:
  void customContextMenuRequested ( const QPoint & pos );
  void removeVar();
  void addVar();
signals:
     void saveSettings(QString, QString,QString, QString,QString,uint,QString,QString,QString);

private:
     Ui::SettingsDialog ui;
};
#endif // PLUGIN_KATEPERLCHECK_H
